@@ -46,6 +46,11 @@
 #include "games-clock.h"
 #include "games-gconf.h"
 
+/* The minimum size for the playing area. Almost completely arbitrary. */
+#define BOARD_MIN_WIDTH 300
+#define BOARD_MIN_HEIGHT 250
+
+
 /*
  * Global Variables
  */
@@ -53,10 +58,11 @@ GtkWidget        *app;
 GtkWidget        *playing_area;
 GtkWidget        *option_dialog = NULL;
 GdkGC            *draw_gc;
+GdkGC            *bg_gc;
+GdkGC            *slot_gc;
 GdkPixmap        *surface;
 GdkPixmap        *moving_card_pixmap;
-GtkObject*       card_deck;
-GdkCardDeckOptions deck_options = NULL;
+gchar            *card_style;
 gboolean         dont_save = FALSE; /* If the game is selected on the
                                      * command line we assume that it is
                                      * special and don't save the state.
@@ -74,12 +80,11 @@ gchar            *game_name;
 gboolean         game_in_progress = FALSE;
 gboolean         game_over;
 gboolean         game_won;
-press_data_type* press_data; 
 
 guint            x_spacing = 5;
 guint            y_spacing = 15;
-guint            x_expanded_offset = 20;
-guint            y_expanded_offset = 20;
+double           x_expanded_offset = 0.21;
+double           y_expanded_offset = 0.21;
 GConfClient      *gconf_client = NULL;
 
 #define DEFAULT_VARIATION "klondike.scm"
@@ -99,37 +104,6 @@ gchar* game_file_to_name (const gchar* file)
   return p;
 }
 
-/*
- * This function assumes that xxx.scm has a HTML help file xxx.html
- * located in the gnome/help/aisleriot/../ dir (nb '_' -> '-' for DocBook).
- */
-static gpointer* game_file_to_help_entry (const gchar* file)
-{
-#if 0
-  GnomeHelpMenuEntry* entry;
-  char* p, *filename;
-  GString* help;
-
-  filename = g_path_get_basename (file);
-  help = g_string_new (filename);
-  g_free (filename);
-
-  if ((p = strrchr (help->str, '.'))) 
-    g_string_truncate (help, p - help->str);
-  for(p = help->str; p = strchr(p, '_'), p && *p;) *p = '-';
-  g_string_append (help, ".html");
-
-  entry = g_new (GnomeHelpMenuEntry, 1);
-  entry->name = g_strdup ("aisleriot");
-  entry->path = g_strdup(help->str);
-
-  g_string_free(help, TRUE);
-  return (gpointer *) entry;
-#else
-  return NULL;
-#endif
-}
-
 void make_title () 
 {
   char *title;
@@ -189,8 +163,8 @@ save_state (GnomeClient *client)
 {
   gconf_client_set_string (gconf_client, "/apps/aisleriot/game_file",
                            game_file, NULL);
-  gconf_client_set_string (gconf_client, "/apps/aisleriot/card_options",
-                           deck_options, NULL);
+  gconf_client_set_string (gconf_client, "/apps/aisleriot/card_style",
+                           card_style, NULL);
 
   return TRUE;
 }
@@ -198,7 +172,6 @@ save_state (GnomeClient *client)
 void new_game (gchar* file, guint *seedp )
 {
   SCM size;
-  gint min_w, min_h;
 
   /* If we're aborting an old game count it as a failure for
    * statistical purposes. */
@@ -209,10 +182,6 @@ void new_game (gchar* file, guint *seedp )
   game_in_progress = FALSE;
 
   if (file && strcmp (file, game_file)) {
-    gchar buf[100];
-    GtkWidget *ms;
-    guint pos;
-
     game_file = file;
 
     /* Explicitly drop any dragged cards. This shouldn't happen, but it
@@ -225,33 +194,14 @@ void new_game (gchar* file, guint *seedp )
      * "clean". */
     eval_installed_file ("sol.scm");
     eval_installed_file (file);
-    if(game_name) {
-      g_snprintf(buf, sizeof (buf), "%s/%s", _("Help"), game_name);
-/*      gnome_app_remove_menus (GNOME_APP (app), buf, 1); */
-      if(rules_help[0].user_data) {
-#if 0
-	GnomeHelpMenuEntry *entry = 
-	  (GnomeHelpMenuEntry *) rules_help[0].user_data;
-	g_free (entry->name);
-	g_free (entry->path);
-	g_free (entry);
-#endif
-      }
-    }
+
     game_name = game_file_to_name (file);
     update_statistics_display ();
+    help_update_game_name (game_name);
 
     if (!dont_save)
       save_state (gnome_master_client ());
 
-    rules_help[0].label = game_name;
-    rules_help[0].user_data = game_file_to_help_entry(file);
-    g_snprintf(buf, sizeof (buf), "%s/%s", _("Help"), _("Aisleriot"));
-
-    ms = gnome_app_find_menu_pos(GNOME_APP (app)->menubar, buf, &pos);
-    if (ms != NULL)
-      gnome_app_fill_menu (GTK_MENU_SHELL(ms), rules_help, 
-			   GNOME_APP (app)->accel_group, TRUE, pos);
     if(option_dialog) {
       gtk_widget_destroy(option_dialog);
       option_dialog = NULL;
@@ -272,9 +222,7 @@ void new_game (gchar* file, guint *seedp )
   size = gh_call0(game_data->start_game_lambda);
   gh_eval_str ("(start-game)");
 
-  min_w = gh_scm2int (gh_car (size))*get_horiz_offset() + 2*get_horiz_start();
-  min_h = gh_scm2int (gh_cadr (size))*get_vert_offset() + 2*get_vert_start();
-  gtk_widget_set_size_request (playing_area, min_w, min_h);
+  set_geometry (gh_scm2double (gh_car (size)), gh_scm2double (gh_cadr (size)));
 
   /* It is possible for some games to not have any moves right from the
    * start. If this happens we redeal. */
@@ -353,16 +301,23 @@ static void create_sol_board ()
   playing_area = gtk_drawing_area_new ();
   gtk_widget_set_events (playing_area, 
 			 gtk_widget_get_events (playing_area) | GAME_EVENTS);
+  /* This only enforces the minimum size. It is actually set using the
+   * window size. */
+  gtk_widget_set_size_request (playing_area, BOARD_MIN_WIDTH, 
+			       BOARD_MIN_HEIGHT);
   
   gnome_app_set_contents (GNOME_APP (app), playing_area);
   
   gtk_widget_realize (playing_area);
 
-  draw_gc = gdk_gc_new(playing_area->window);
-  if (get_background_pixmap ())
-    gdk_gc_set_tile (draw_gc, get_background_pixmap());
-  gdk_gc_set_fill (draw_gc, GDK_TILED);
+  draw_gc = gdk_gc_new (playing_area->window);
   
+  bg_gc = gdk_gc_new (playing_area->window);
+  if (get_background_pixmap ())
+    gdk_gc_set_tile (bg_gc, get_background_pixmap());
+  gdk_gc_set_fill (bg_gc, GDK_TILED);  
+
+  slot_gc = gdk_gc_new (playing_area->window);
 
   g_signal_connect (GTK_OBJECT(playing_area),"button_release_event",
 		      GTK_SIGNAL_FUNC (button_release_event), NULL);
@@ -370,6 +325,8 @@ static void create_sol_board ()
 		      GTK_SIGNAL_FUNC (motion_notify_event), NULL);
   g_signal_connect (GTK_OBJECT (playing_area), "button_press_event",
 		      GTK_SIGNAL_FUNC (button_press_event), NULL);
+  g_signal_connect (GTK_OBJECT (playing_area), "configure_event",
+		      GTK_SIGNAL_FUNC (configure_event), NULL);
 }
 
 void quit_app (GtkMenuItem *menuitem)
@@ -384,32 +341,21 @@ void quit_app (GtkMenuItem *menuitem)
 
 static void create_main_window ()
 {
+  GConfClient * gconf_client = gconf_client_get_default ();
+  gint width, height;
+
+  width = gconf_client_get_int (gconf_client, WIDTH_GCONF_KEY, NULL);
+  height = gconf_client_get_int (gconf_client, HEIGHT_GCONF_KEY, NULL);
+
   app = gnome_app_new ("aisleriot", _("Aisleriot"));
+  gtk_window_set_default_size (GTK_WINDOW (app), width, height);
 
   gtk_widget_realize (app);
 
   g_signal_connect (GTK_OBJECT (app), "delete_event", 
 		      GTK_SIGNAL_FUNC (quit_app), NULL);
   g_signal_connect (GTK_OBJECT (app), "configure_event",
-		      GTK_SIGNAL_FUNC (configure_event), NULL);
-}
-
-static void create_press_data ()
-{
-  GdkWindowAttr attributes;
-
-  attributes.wclass = GDK_INPUT_OUTPUT;
-  attributes.window_type = GDK_WINDOW_CHILD;
-  attributes.event_mask = 0;
-  attributes.width = get_card_width();
-  attributes.height = get_card_height();
-  attributes.colormap = gdk_drawable_get_colormap (GDK_DRAWABLE(playing_area->window));
-  attributes.visual = gdk_drawable_get_visual (GDK_DRAWABLE(playing_area->window));
-  
-  press_data = g_malloc0(sizeof(press_data_type));
-  press_data->moving_cards = gdk_window_new(playing_area->window, &attributes,
-					    (GDK_WA_VISUAL | GDK_WA_COLORMAP));
-  press_data->status = 0;
+		      GTK_SIGNAL_FUNC (app_configure_event), NULL);
 }
 
 gchar* start_game;
@@ -425,7 +371,7 @@ static void main_prog(int argc, char *argv[])
 
   create_main_window ();
 
-  load_pixmaps (app, deck_options);
+  load_pixmaps ();
 
   create_sol_board ();
   create_menus ();
@@ -454,13 +400,12 @@ static void main_prog(int argc, char *argv[])
                              "/apps/aisleriot/show_toolbar", NULL))
     toolbar_hide();
 
-  create_press_data ();
+  create_press_data (playing_area->window);
 
   gtk_main ();
 
   free_pixmaps();
   g_object_unref (surface);
-  g_object_unref (press_data->moving_cards);
 }
 
 static void
@@ -469,9 +414,9 @@ retrieve_state (GnomeClient *client)
   start_game = games_gconf_get_string (gconf_client,
                                        "/apps/aisleriot/game_file",
                                        "klondike.scm");
-  deck_options = games_gconf_get_string (gconf_client,
-                                         "/apps/aisleriot/card_options",
-                                         "beige.png bonded.png gnome.png bold-09x14.png knuth-09x10.png knuth-18x21.png knuth-21x25.png");
+  card_style = games_gconf_get_string (gconf_client,
+				       "/apps/aisleriot/card_style",
+				       "bonded-new.png");
 }
 
 static int
