@@ -684,6 +684,32 @@ tilepos fourbridges_map [MAX_TILES] = {
 };
 
 
+
+tilepos ziggurat_map [MAX_TILES] = {
+ {10, 8, 5}, {12, 8, 5}, {14, 8, 5}, {16, 8, 5}, {18, 8, 5},
+ {10, 6, 5}, {12, 6, 5}, {14, 6, 5}, {16, 6, 5}, {18, 6, 5},
+ {8, 8, 4}, {10, 8, 4}, {12, 8, 4}, {14, 8, 4}, {16, 8, 4}, {18, 8, 4}, {20, 8, 4},
+ {8, 6, 4}, {10, 6, 4}, {12, 6, 4}, {14, 6, 4}, {16, 6, 4}, {18, 6, 4}, {20, 6, 4},
+ {6, 8, 3}, {8, 8, 3}, {10, 8, 3}, {12, 8, 3}, {14, 8, 3}, {16, 8, 3}, {18, 8, 3}, {20, 8, 3}, {22, 8, 3},
+ {6, 6, 3}, {8, 6, 3}, {10, 6, 3}, {12, 6, 3}, {14, 6, 3}, {16, 6, 3}, {18, 6, 3}, {20, 6, 3}, {22, 6, 3},
+ {6, 13,  2},  {22, 13,  2}, {14, 12,  2},
+ {4, 8,  2}, {6, 8, 2}, {8, 8, 2}, {20, 8, 2}, {22, 8, 2}, {24, 8, 2},
+ {4, 6,  2}, {6, 6, 2}, {8, 6, 2}, {20, 6, 2}, {22, 6, 2}, {24, 6, 2},
+ {14, 2,  2}, {6, 1,  2}, {22, 1,  2},
+ {6, 13,  1},  {22, 13,  1}, {13, 12,  1}, {15, 12,  1},
+ {2, 8, 1}, {4, 8,  1}, {6, 8, 1}, {22, 8,  1}, {24, 8,  1}, {26, 8,  1},
+ {2, 6, 1}, {4, 6,  1}, {6, 6, 1}, {22, 6,  1}, {24, 6,  1}, {26, 6,  1},
+ {13, 2,  1}, {15, 2,  1}, {6, 1,  1}, {22, 1,  1},
+ {0,14,0}, {5, 14,  0}, {7, 14,  0}, {9, 14,  0}, {11, 14,  0}, {13, 14, 0}, {15, 14,  0}, {17, 14, 0}, {19, 14,  0}, {21, 14, 0}, {23, 14,  0}, {28,14,0},
+ {0,12,0}, {5, 12,  0},{13,12,0},{15,12,0},{23, 12,  0},{28,12,0},
+ {0,10,0}, {5, 10,  0}, {13, 10, 0}, {15, 10,  0}, {23, 10,  0},{28,10,0},
+ {0, 8,  0}, {2, 8, 0}, {4, 8,  0}, {6, 8, 0}, {22, 8,  0}, {24, 8,  0}, {26, 8,  0}, {28, 8,  0},
+ {0, 6,  0}, {2, 6, 0}, {4, 6,  0}, {6, 6, 0}, {22, 6,  0}, {24, 6,  0}, {26, 6,  0}, {28, 6,  0},
+ {0,4,0}, {5, 4,  0}, {13, 4,  0}, {15, 4,  0}, {23, 4,  0},{28,4,0},
+ {0,2,0},{5, 2,  0}, {13,2,0},{15,2,0},{23, 2,  0},{28,2,0},
+ {0,0,0},{5, 0,  0}, {7, 0,  0}, {9, 0,  0}, {11, 0,  0}, {13, 0, 0}, {15, 0,  0}, {17, 0,  0}, {19, 0,  0}, {21, 0, 0}, {23, 0,  0},{28,0,0},
+};
+
 tilepos *pos = 0;
 
 tile tiles[MAX_TILES];
@@ -695,7 +721,7 @@ gint sequence_number;
 
 guint current_seed, next_seed;
 
-gint windowwidth, windowheight;
+static gint windowwidth, windowheight;
 
 GList * tileset_list = NULL;
 
@@ -719,7 +745,7 @@ struct _maps
         { N_("Cloud"), "cloud", cloud_map },
         { N_("Red Dragon"), "dragon", reddragon_map },
         { N_("Four Bridges"), "bridges", fourbridges_map },
-
+	{ N_("The Ziggurat"), "ziggurat", ziggurat_map },
 };
 
 
@@ -881,7 +907,7 @@ static void set_menus_sensitive (void)
 	state = game_over != GAME_WON;
 	gtk_widget_set_sensitive (gamemenu[3].widget, state);
 	gtk_widget_set_sensitive (toolbar_uiinfo[2].widget, state);
-
+	
 	/* Hint */
 	state = moves_left > 0;
 	gtk_widget_set_sensitive (gamemenu[7].widget, state);
@@ -1388,7 +1414,7 @@ you_won (void)
 	
         game_over = GAME_WON;
 
-        seconds = GAMES_CLOCK (chrono)->stopped;
+        seconds = games_clock_get_seconds (GAMES_CLOCK (chrono));
 
         score = (seconds / 60) * 1.0 + (seconds % 60) / 100.0;
 	pos = gnome_score_log (score, score_current_mapset, FALSE);
@@ -1552,7 +1578,6 @@ void
 hint_callback (GtkWidget *widget, gpointer data)
 {
         gint i, j, free=0, type ;
-        time_t seconds;
 
         if (paused || game_over)
                 return;
@@ -1598,10 +1623,7 @@ hint_callback (GtkWidget *widget, gpointer data)
 	timer = g_timeout_add (250, (GSourceFunc) hint_timeout, NULL);
                 
 	/* 30s penalty */
-	games_clock_stop (GAMES_CLOCK(chrono));
-	seconds = GAMES_CLOCK(chrono)->stopped;
-	games_clock_set_seconds(GAMES_CLOCK(chrono), (gint) (seconds+30));
-	games_clock_start (GAMES_CLOCK(chrono));
+	games_clock_add_seconds(GAMES_CLOCK(chrono), 30);
 }
 
 static void
@@ -1622,6 +1644,8 @@ about_callback (GtkWidget *widget, gpointer data)
 		"Michael Meeks",
                 "Philippe Chavin",
 		"Callum McKenzie",
+		"",
+		N_("Maps:"),
 		"Rexford Newbould",
 		"Krzysztof Foltman",
 		"",
@@ -1658,7 +1682,7 @@ about_callback (GtkWidget *widget, gpointer data)
 	}
 	
 	about = gnome_about_new (_("GNOME Mahjongg"), VERSION,
-				 "Copyright \xc2\xa9 1998-2003 Free Software "
+				 "Copyright \xc2\xa9 1998-2004 Free Software "
 				 "Foundation, Inc.",
 				 _("GNOME version of the Eastern tile game, "
 				   "Mahjongg."),
@@ -1680,16 +1704,30 @@ about_callback (GtkWidget *widget, gpointer data)
 void
 pause_callback (void)
 {
+	static gboolean noloops = FALSE;
+
+	/* The calls to set the menu bar toggle-button will
+         * trigger another callback, which will trigger another
+	 * callback ... this must be stopped. */
+	if (noloops)
+                return;
+
+	noloops = TRUE;
         paused = !paused;
 	draw_all_tiles ();
         if (paused) {
+		gtk_toggle_button_set_active (
+                  GTK_TOGGLE_BUTTON (toolbar_uiinfo[2].widget), TRUE);
                 games_clock_stop (GAMES_CLOCK (chrono));
-                message(_("... Game paused ..."));
+                message(_("Game paused"));
         }
         else {
-                message ("");
+		gtk_toggle_button_set_active (
+                  GTK_TOGGLE_BUTTON (toolbar_uiinfo[2].widget), FALSE);
                 games_clock_start (GAMES_CLOCK(chrono));
+                message ("");
         }
+	noloops = FALSE;
 }
 
 void ensure_pause_off (void)
@@ -1732,11 +1770,11 @@ init_game (void)
 void
 confirm_action (GtkWidget *widget, gpointer data)
 {
-	switch ((gint)data) {
+	switch (GPOINTER_TO_INT(data)) {
 	case NEW_GAME:
 	case NEW_GAME_WITH_SEED:
 		ensure_pause_off ();
-		new_game ((gint)data == NEW_GAME);
+		new_game (GPOINTER_TO_INT(data) == NEW_GAME);
 		break;
 	case RESTART_GAME:
 		restart_game ();
@@ -2054,7 +2092,6 @@ new_game (gboolean re_seed)
 void
 shuffle_tiles_callback (GtkWidget *widget, gpointer data)
 {
-        time_t seconds;
 	gboolean ok;
 
         if (paused || game_over == GAME_DEAD || game_over == GAME_WON) return;
@@ -2085,11 +2122,9 @@ shuffle_tiles_callback (GtkWidget *widget, gpointer data)
                 game_over = GAME_RUNNING;
 
                 /* 60s penalty */
-                games_clock_stop (GAMES_CLOCK(chrono));
-                seconds = GAMES_CLOCK(chrono)->stopped;
-                games_clock_set_seconds(GAMES_CLOCK(chrono), (gint) (seconds+60));
-                games_clock_start (GAMES_CLOCK(chrono));
+                games_clock_add_seconds(GAMES_CLOCK(chrono), 60);
 
+		update_moves_left ();
 		/* Disable undo/redo after a shuffle. */
 		sequence_number = 1;
 		clear_undo_queue ();
